package italo.comecome;

import italo.comecome.a_estrela.CaminhoAEstrela;
import italo.comecome.audio.Audio;
import italo.comecome.gui.ContainerJogo;
import italo.comecome.gui.desenho.DesenhoTabuleiro;
import italo.comecome.gui.figura.Imagem;
import italo.comecome.logica.Fase;
import italo.comecome.logica.FaseListener;
import italo.comecome.logica.Tabuleiro;
import italo.comecome.personagem.ComeCome;
import italo.comecome.personagem.Monstrinho;
import italo.comecome.personagem.Personagem;
import italo.comecome.personagem.Posicao;
import italo.comecome.personagem.util.PersonagemUtil;
import italo.comecome.thread.Gerenciador;
import italo.comecome.utilitarios.Utilitarios;

import java.util.Map;

public class HandleFase implements FaseListener{

	protected ContainerJogo container;	
	protected DesenhoTabuleiro desenhoTabuleiro;
	protected int espera = 100;	
	
	public HandleFase(ContainerJogo container, DesenhoTabuleiro desenhoTabuleiro) {
		this.container = container;	
		this.desenhoTabuleiro = desenhoTabuleiro;
	}
		
	public void executar(Fase fase) {
		// no necessrio por enquanto
	}
	
	public void iniciar(final Fase fase) {		
		fase.setExecutando(true);		
		iniciarAtributosPersonagens(fase);
		iniciarTabuleiroGUI(fase);	
		container.playAudioClip(Audio.CAMINHO_FASE+fase.getNumero()+Audio.EXT);
		iniciarFiguras(fase);
		iniciarPersonagens(fase);		
	}
	
	public void reiniciar(final Fase fase) {
		fase.setExecutando(true);		
		iniciarAtributosPersonagens(fase);
		iniciarFiguras(fase);
		iniciarPersonagens(fase);		
	}
		
	public void finalizar(Fase fase) {		
		fase.setExecutando(false);
		fase.finalizarMonstrinhos();
	}
	
	public void sairDaCasaInicio(Fase fase, Monstrinho monstrinho) {
		int indiceMonstrinho = fase.getIndiceMonstrinho(monstrinho);
		sairDaCasaInicio(fase, indiceMonstrinho);		
	} 
	
	public void entrarNaCasa(Fase fase, Monstrinho monstrinho) {
		int indiceMonstrinho = fase.getIndiceMonstrinho(monstrinho);
		entrarNaCasa(fase, indiceMonstrinho);
	}
	
	public void sairDaCasa(Fase fase, Monstrinho monstrinho) {
		int indiceMonstrinho = fase.getIndiceMonstrinho(monstrinho);
		sairDaCasa(fase, indiceMonstrinho);
	}
		
	public void mover(Fase fase, Personagem personagem) {
		Tabuleiro tabuleiro = fase.getTabuleiro();
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		int x1 = tabuleiro.getX1();
		int y1 = tabuleiro.getY1();
		int estadoMonstrinho = (personagem instanceof Monstrinho)?((Monstrinho)personagem).getEstado():Monstrinho.FORTE;
				
		String nomeStr = personagem.getNome();
		String idStr = String.valueOf(personagem.getId());
		String estadoStr = PersonagemUtil.getEstado(estadoMonstrinho);
		String direcaoStr = PersonagemUtil.getDirecao(personagem.getDirecao());		
		Imagem img = container.getFigura().novaImagem(nomeStr, idStr, direcaoStr,	estadoStr); 			
		img.setDim(larguraCelula, alturaCelula);
		img.setPosicao(x1+personagem.getPosicao().getX(), y1+personagem.getPosicao().getY());
		if(personagem instanceof ComeCome) {
			desenhoTabuleiro.getImagensComeCome().put(personagem.getId(), img);
		} else {
			desenhoTabuleiro.getImagensMonstrinho().put(personagem.getId(), img);
			desenhoTabuleiro.setCaminho(personagem.getId(), ((Monstrinho)personagem).getCaminhoAEstrela());
		}		
		desenhoTabuleiro.desenharFiguras(fase.getBolinhas(), fase.getBolinhasGrandes());		
	}
			
	public void mostrarCaminhoOtimo(Fase fase, Monstrinho monstrinho, boolean b) {
		desenhoTabuleiro.mostrarCaminho(monstrinho.getId(), b);
	}
		
	public void comeuBolinha(Fase fase) {
		if(fase.getNBolinhas()%3==0) 
			container.playAudioClip(Audio.COMEU);
	}
	
	public void comeuBolinhaGrande(Fase fase) {
		container.playAudioClip(Audio.COMEUBG);		
		Personagem[] monstrinhos = fase.getMonstrinhos();
		for(int i = 0; i < monstrinhos.length; i++) {
			Monstrinho monstrinho = ((Monstrinho)monstrinhos[i]);
			if(monstrinho.getEstado() == Monstrinho.FORTE)
				monstrinho.setEstado(Monstrinho.FRACO);				
		}
	}
	
	public void comeuMonstrinho(Fase fase) {
		container.playAudioClip(Audio.OOPS);
	}
		
	private void iniciarAtributosPersonagens(Fase fase) {
		fase.getComeCome().iniciar();
		Personagem[] monstrinhos = fase.getMonstrinhos();
		for(Personagem p:monstrinhos)
			p.iniciar();
	}
	
	private void iniciarPersonagens(Fase fase) {		
		Gerenciador.getInstance().iniciarNovoComeCome(fase.getComeCome());
		Gerenciador.getInstance().iniciarNovosMonstrinhos(fase.getMonstrinhos());		
		Gerenciador.getInstance().startComeCome();
		Personagem[] monstrinhos = fase.getMonstrinhos();
		for(int i = 0; i < monstrinhos.length && fase.isExecutando(); i++) {
			Monstrinho monstrinho = ((Monstrinho)monstrinhos[i]);
			sairDaCasaInicio(fase, monstrinho);
			Gerenciador.getInstance().startMonstrinho(i);	
		}	
	}

	private void sairDaCasaInicio(Fase fase, int i) {
		Personagem[] monstrinhos = fase.getMonstrinhos();	
		int[][] direcaoMonstrinhos = fase.getDirecoesIniciaisMonstrinhos();
		if(i < direcaoMonstrinhos.length) {			
			int[] direcaoM = direcaoMonstrinhos[i]; 
			for(int j = 0; j < direcaoM.length && fase.isExecutando(); j++) {
				Monstrinho monstrinho = (Monstrinho)monstrinhos[i];
				while(!monstrinho.isFim() && monstrinho.isParar());
				if(monstrinho.isFim())
					continue;
				Utilitarios.sleep(espera);		
				monstrinho.setDirecao(direcaoM[j]);
				monstrinho.setNovaDirecao(direcaoM[j]);			
				monstrinho.mover();
			}
		}
	}
			
	private void entrarNaCasa(Fase fase, int i) {
		Tabuleiro tabuleiro = fase.getTabuleiro();
		Monstrinho monstrinho = (Monstrinho)fase.getMonstrinhos()[i];
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		int x = monstrinho.getPosicao().getX();
		int y = monstrinho.getPosicao().getY();
		boolean podeEntrar = fase.podeEntrarNaCasa(x/larguraCelula, y/alturaCelula);
		if(fase.isPosicaoCelula(x, y) && podeEntrar) {
			for(int j = 0; j < tabuleiro.getFatorMiniCelula() && fase.isExecutando(); j++) {
				while(monstrinho.isParar());				
				Utilitarios.sleep(espera);
				monstrinho.setDirecao(Personagem.BAIXO);
				monstrinho.setNovaDirecao(Personagem.BAIXO);
				monstrinho.mover();				
			}
		}
	}
			
	private void sairDaCasa(Fase fase, int i) {
		Tabuleiro tabuleiro = fase.getTabuleiro();
		Monstrinho monstrinho = (Monstrinho)fase.getMonstrinhos()[i];
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		int x = monstrinho.getPosicao().getX();
		int y = monstrinho.getPosicao().getY();
		boolean podeSair = fase.podeSairDaCasa(x/larguraCelula, y/alturaCelula);
		if(fase.isPosicaoCelula(x, y) && podeSair) {
			for(int j = 0; j < tabuleiro.getFatorMiniCelula() && fase.isExecutando(); j++) {
				while(monstrinho.isParar());									
				Utilitarios.sleep(espera);
				monstrinho.setDirecao(Personagem.CIMA);
				monstrinho.setNovaDirecao(Personagem.CIMA);			
				monstrinho.mover();				
			}
		}		
	}
			
	private void iniciarTabuleiroGUI(Fase fase) {		
		String caminhoImagem = fase.getCaminhoImagemTabuleiro();
		desenhoTabuleiro.setCaminhoImagem(caminhoImagem);
		container.getContainer().setVisible(true);
	}
	
	private void iniciarFiguras(Fase fase) {
		desenhoTabuleiro.novaListaImagensComeCome();
		desenhoTabuleiro.novaListaImagensMonstrinho();
		desenhoTabuleiro.novaListaCaminhoAEstrela();
		Map<Integer, Imagem> imagensComeCome = desenhoTabuleiro.getImagensComeCome();
		Map<Integer, Imagem> imagensMonstrinho = desenhoTabuleiro.getImagensMonstrinho();
		iniciarFiguraComeCome(fase, imagensComeCome);
		iniciarFigurasMonstrinhos(fase, imagensMonstrinho);
		desenhoTabuleiro.desenharFiguras(fase.getBolinhas(), fase.getBolinhasGrandes());	
	}
	
	private void iniciarFiguraComeCome(Fase fase, Map<Integer, Imagem> imagens) {
		Tabuleiro tabuleiro = fase.getTabuleiro();
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		int x1 = tabuleiro.getX1();
		int y1 = tabuleiro.getY1();
		int[][] posicoesIniciaisComeCome = fase.getPosicoesIniciaisComecome();
		Personagem comecome = fase.getComeCome();
		int xInicial = posicoesIniciaisComeCome[0][0]*larguraCelula;
		int yInicial = posicoesIniciaisComeCome[0][1]*alturaCelula;
		comecome.setPosicao(new Posicao(xInicial, yInicial));
		Imagem fComeCome = container.getFigura().novaImagem(
				comecome.getNome(), 
				String.valueOf(comecome.getId()),
				PersonagemUtil.getDirecao(comecome.getDirecao()), 
				PersonagemUtil.getEstado(Monstrinho.FORTE)); 
		fComeCome.setDim(larguraCelula, alturaCelula);
		fComeCome.setPosicao(x1+comecome.getPosicao().getX(), y1+comecome.getPosicao().getY());
		imagens.put(comecome.getId(), fComeCome);	
		
	}
	
	private void iniciarFigurasMonstrinhos(Fase fase, Map<Integer, Imagem> imagens) {
		Tabuleiro tabuleiro = fase.getTabuleiro();
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		int x1 = tabuleiro.getX1();
		int y1 = tabuleiro.getY1();
		int[][] posicoesIniciaisMonstrinhos = fase.getPosicoesIniciaisMonstrinhos();
		Personagem[] monstrinhos = fase.getMonstrinhos();
		for(int i = 0; i < monstrinhos.length; i++) {
			Monstrinho monstrinho = (Monstrinho) monstrinhos[i];
			int xInicial = posicoesIniciaisMonstrinhos[i][0]*larguraCelula;
			int yInicial = posicoesIniciaisMonstrinhos[i][1]*alturaCelula;
			monstrinho.setPosicao(new Posicao(xInicial, yInicial)); 
			Imagem fMonstrinho = container.getFigura().novaImagem(
					monstrinho.getNome(), 
					String.valueOf(monstrinho.getId()),
					PersonagemUtil.getDirecao(monstrinho.getDirecao()), 
					PersonagemUtil.getEstado(Monstrinho.FORTE)); 
			fMonstrinho.setDim(larguraCelula, alturaCelula);
			fMonstrinho.setPosicao(x1+monstrinho.getPosicao().getX(), y1+monstrinho.getPosicao().getY());
			imagens.put(monstrinho.getId(), fMonstrinho);
			desenhoTabuleiro.getCaminhosAEstrela().put(monstrinho.getId(), new CaminhoAEstrela());
			desenhoTabuleiro.setCorCaminhoAEstrela(monstrinho.getId(), Utilitarios.getCorAleatoria());
		}				
	}
	
}
