package italo.comecome;

import italo.comecome.audio.Audio;
import italo.comecome.entrada.Entrada;
import italo.comecome.gui.ContainerJogo;
import italo.comecome.gui.desenho.DesenhoMenu;
import italo.comecome.gui.desenho.DesenhoTabuleiro;
import italo.comecome.gui.figura.Figura;
import italo.comecome.logica.Fase;
import italo.comecome.logica.Jogo;
import italo.comecome.logica.JogoListener;
import italo.comecome.logica.Tabuleiro;
import italo.comecome.personagem.ComeCome;
import italo.comecome.personagem.Monstrinho;
import italo.comecome.personagem.Personagem;
import italo.comecome.thread.Gerenciador;
import italo.comecome.truque.Truque;
import italo.comecome.utilitarios.Utilitarios;

public class HandlerJogo implements JogoListener {		
	
	private ContainerJogo container;
	private Entrada entrada;
	private DesenhoTabuleiro[] desenhosTabuleiro;
	private DesenhoMenu desenhoMenu;
	private HandleFase[] handleFases;
	private HandlerEntradaJogo handlerEntradaJogo;
	private HandlerEntradaMenuJogo handlerEntradaMenuJogo;
	private HandlerTruqueAtravessarParedes handlerTruqueAtravessarParedes;
	private HandlerTruqueMudarDeFase handlerTruqueMudarDeFase;
	
	private boolean jogoParado = false;
	
	public HandlerJogo(ContainerJogo applet, Entrada entrada) {
		this.container = applet;	
		this.entrada = entrada;
	}
					
	public void iniciar(Jogo jogo) {							
		configurarFases(jogo);
		desenhoMenu = new DesenhoMenu(container);		
		handlerEntradaJogo = new HandlerEntradaJogo(jogo);
		handlerEntradaMenuJogo = new HandlerEntradaMenuJogo(entrada, desenhoMenu, jogo, container);
		handlerTruqueAtravessarParedes = new HandlerTruqueAtravessarParedes(jogo, container);
		handlerTruqueMudarDeFase = new HandlerTruqueMudarDeFase(jogo);
		entrada.addTecladoListener(handlerEntradaMenuJogo);
		Truque truque = jogo.getTruquesJogo().get(Jogo.CHAVE_TRUQUE_ATRAVESSAR_PAREDES);
		Truque truque2 = jogo.getTruquesJogo().get(Jogo.CHAVE_TRUQUE_MUDAR_DE_FASE);		
		truque.addTruqueListener(handlerTruqueAtravessarParedes);
		truque2.addTruqueListener(handlerTruqueMudarDeFase);
	}
	
	public void startar(Jogo jogo) {		
		Gerenciador.getInstance().iniciarNovoJogo(jogo);
		Gerenciador.getInstance().startJogo();
	}
		
	public void executar(Jogo jogo) {	
		addListeners(jogo);	
		while(!jogo.isFimJogo()) {			
			executarFaseAtual(jogo);
			Gerenciador.getInstance().waitJogo();			
		}	
		finalizar(jogo);
	}
				
	public void finalizar(Jogo jogo) {
		removeListeners(jogo);		
		jogo.iniciar();
	}
	
	public void sair(Jogo jogo) {
		entrada.removeEntradaListener();
		container.finalizar();
	}
		
	public void continuarExecucao(Jogo jogo) {
		if(jogo.getFaseAtual().isExecutando()) {
			Personagem comecome = jogo.getFaseAtual().getComeCome();
			Personagem[] monstrinhos = jogo.getFaseAtual().getMonstrinhos();
			((ComeCome)comecome).continuarExecucao();
			for(Personagem monstrinho:monstrinhos)
				((Monstrinho)monstrinho).continuarExecucao();
			desenhosTabuleiro[jogo.getIndiceFaseAtual()].apagarMensagem();
		}
	}
	
	public void pararExecucao(Jogo jogo) {	
		if(jogo.getFaseAtual().isExecutando()) {
			Personagem comecome = jogo.getFaseAtual().getComeCome();
			Personagem[] monstrinhos = jogo.getFaseAtual().getMonstrinhos();
			((ComeCome)comecome).pararExecucao();
			for(Personagem monstrinho:monstrinhos)
				((Monstrinho)monstrinho).pararExecucao();
			String msg = Figura.MENSAGEM_PAUSA;
			int larguraMsg = jogo.getFaseAtual().getTabuleiro().getLarguraTabuleiro()*1/3;
			int alturaMsg = jogo.getFaseAtual().getTabuleiro().getAlturaTabuleiro()*1/5;
			DesenhoTabuleiro desenhoTabuleiro = desenhosTabuleiro[jogo.getIndiceFaseAtual()];
			desenhoTabuleiro.desenharMensagem(msg, larguraMsg, alturaMsg);
		}
	}
				
	public void perdeu(Jogo jogo) {			
		jogo.setNVidasAtual(jogo.getNVidasAtual()-1);		
		int nVidas = jogo.getNVidasAtual();
		if(nVidas > 0) {			
			DesenhoTabuleiro desenhoTabuleiro = desenhosTabuleiro[jogo.getIndiceFaseAtual()];
			String msg = Figura.CAMINHO_MENSAGENS_VIDAS+nVidas+Figura.EXT;
			int larguraMsg = jogo.getFaseAtual().getTabuleiro().getLarguraTabuleiro()*3/4;
			int alturaMsg = jogo.getFaseAtual().getTabuleiro().getAlturaTabuleiro()*1/5;
			desenhoTabuleiro.desenharMensagem(msg, larguraMsg, alturaMsg);
			container.playAudioClip(Audio.OOH);
			Utilitarios.sleep(500);			
			container.playAudioClip(Audio.CAMINHO_VIDA+nVidas+Audio.EXT);
			Utilitarios.sleep(3500);
			desenhoTabuleiro.apagarMensagem();
			jogo.getFaseAtual().reiniciar();			
		} else {					
			jogo.gameover();
			while(!jogoParado);	
			Gerenciador.getInstance().notificarJogo();				
		}		
	}

	public void mudarDeFase(Jogo jogo, int nFase) {
		jogo.setIndiceFaseAtual(nFase);
		if(jogo.isFimJogo()) {
			jogo.zerou();
		} else {
			while(!jogoParado);	
			Gerenciador.getInstance().notificarJogo();
		}
	}
	
	public void passou(Jogo jogo) {
		DesenhoTabuleiro desenhoTabuleiro = desenhosTabuleiro[jogo.getIndiceFaseAtual()];
		String msg = Figura.MENSAGEM_PASSOU;
		int larguraMsg = jogo.getFaseAtual().getTabuleiro().getLarguraTabuleiro()*3/4;
		int alturaMsg = jogo.getFaseAtual().getTabuleiro().getAlturaTabuleiro()*1/5;
		desenhoTabuleiro.desenharMensagem(msg, larguraMsg, alturaMsg);	
		container.playAudioClip(Audio.PASSOU);
		Utilitarios.sleep(3000);
		desenhoTabuleiro.apagarMensagem();
		mudarDeFase(jogo, jogo.getIndiceFaseAtual()+1);
	}
	
	public void zerou(Jogo jogo) {
		DesenhoTabuleiro desenhoTabuleiro = desenhosTabuleiro[jogo.getIndiceFaseAtual()-1];
		String msg = Figura.MENSAGEM_ZEROU;
		int larguraMsg = jogo.getFaseAtual().getTabuleiro().getLarguraTabuleiro()*3/4;
		int alturaMsg = jogo.getFaseAtual().getTabuleiro().getAlturaTabuleiro()*1/2;
		desenhoTabuleiro.desenharMensagem(msg, larguraMsg, alturaMsg);
		container.playAudioClip(Audio.WOODY);
		Utilitarios.sleep(3000);
		container.playAudioClip(Audio.ZEROU);
		Utilitarios.sleep(3000);
		desenhoTabuleiro.apagarMensagem();
		while(!jogoParado);	
		Gerenciador.getInstance().notificarJogo();		
	}
	
	public void gameover(Jogo jogo) {
		DesenhoTabuleiro desenhoTabuleiro = desenhosTabuleiro[jogo.getIndiceFaseAtual()];
		String msg = Figura.MENSAGEM_GAMEOVER;
		int larguraMsg = jogo.getFaseAtual().getTabuleiro().getLarguraTabuleiro()*3/4;
		int alturaMsg = jogo.getFaseAtual().getTabuleiro().getAlturaTabuleiro()*1/2;
		desenhoTabuleiro.desenharMensagem(msg, larguraMsg, alturaMsg);		
		container.playAudioClip(Audio.RISADAMALIGNA);
		Utilitarios.sleep(5000);
		container.playAudioClip(Audio.GAMEOVER);
		Utilitarios.sleep(3000);
		desenhoTabuleiro.apagarMensagem();
	}
	
	private void executarFaseAtual(Jogo jogo) {
		jogoParado = false;	
		handlerEntradaJogo.iniciar();
		desenhosTabuleiro[jogo.getIndiceFaseAtual()].apagarMensagem();
		jogo.getFaseAtual().iniciar();
		jogoParado = true;
	}
	
	private void configurarFases(Jogo jogo) {
		Fase[] fases = jogo.getFases();
		handleFases = new HandleFase[fases.length];
		desenhosTabuleiro = new DesenhoTabuleiro[fases.length];
		for(int i = 0; i < fases.length; i++) { 
			Tabuleiro tabuleiro = fases[i].getTabuleiro(); 		
			int x1 = tabuleiro.getX1();
			int y1 = tabuleiro.getY1();
			int largura = tabuleiro.getLarguraTabuleiro();
			int altura = tabuleiro.getAlturaTabuleiro();
			int larguraCelula = tabuleiro.getLarguraCelula();
			int alturaCelula = tabuleiro.getAlturaCelula();			
			desenhosTabuleiro[i] = new DesenhoTabuleiro(container, x1, y1, largura, altura, larguraCelula, alturaCelula);
			handleFases[i] = new HandleFase(container, desenhosTabuleiro[i]);
		}		
	}
		
	private void addListeners(Jogo jogo) {	
		Fase[] fases = jogo.getFases();
		for(int i = 0; i < fases.length; i++) 
			fases[i].addFaseListener(handleFases[i]);	
		entrada.addTecladoListener(handlerEntradaJogo);
	}
		
	private void removeListeners(Jogo jogo) {	
		Fase[] fases = jogo.getFases();
		for(int i = 0; i < fases.length; i++) 
			fases[i].removeFaseListener(handleFases[i]);	
		entrada.removeTecladoListener(handlerEntradaJogo);
	}
		
}
