package italo.comecome;


import italo.comecome.audio.Audio;
import italo.comecome.conf.Conf;
import italo.comecome.entrada.Entrada;
import italo.comecome.gui.ContainerJogo;
import italo.comecome.gui.PainelPersonalizado;
import italo.comecome.gui.desenho.CarregamentoIT;
import italo.comecome.gui.desenho.Desenho;
import italo.comecome.gui.desenho.DesenhoCarregamento;
import italo.comecome.gui.desenho.DesenhoConcreto;
import italo.comecome.gui.figura.Figura;
import italo.comecome.logica.Jogo;

import java.awt.Color;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.Image;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JApplet;

public class JogoApplet extends JApplet implements ContainerJogo, Runnable {
	
	private static final long serialVersionUID = 1L;
	
	public final static int LARGURA_PADRAO = 720;
	public final static int ALTURA_PADRAO = 540;
	public final static String CAMINHO_IMAGEM_ABERTURA = "img/abertura/1.gif";
	public final static Color COR_FUNDO = Color.BLACK;
	
	private Desenho fundo = null;	
	private PainelPersonalizado painelPersonalizado = null;
		
	private Audio audio;
	private Figura figura;
	private Jogo jogo;	
		
	public JogoApplet() {					
		fundo = new DesenhoConcreto(this, CAMINHO_IMAGEM_ABERTURA);		
		painelPersonalizado = new PainelPersonalizado(fundo);
		painelPersonalizado.setOpaque(false);
		setFocusable(true);		
		setBackground(COR_FUNDO);
		setLayout(new GridLayout());
		setContentPane(painelPersonalizado);		
	}
	
	public void run() {
		setVisible(true);
		
		jogo = new Jogo();
		figura = new Figura(this);
		audio = new Audio(this);	
	
		Entrada entrada = new Entrada(this);		
		HandlerJogo handlerJogo = new HandlerJogo(this, entrada);														
		
		Conf.getInstance().configurarJogo(jogo, figura, audio, this.getSize().width, this.getSize().height);
		DesenhoCarregamento desenhoCarregamento = new DesenhoCarregamento(this);
		desenhoCarregamento.carregar();

		jogo.addJogoListener(handlerJogo);
		entrada.addEntradaListener();				
		
		jogo.iniciar();			
	}
	
	public void redimensionar(String larguraStr, String alturaStr) {
		try {
			int largura = Integer.parseInt(larguraStr);
			int altura = Integer.parseInt(alturaStr);
			setSize(largura, altura);
		} catch (NumberFormatException e) {
			setSize(LARGURA_PADRAO, ALTURA_PADRAO);
		} catch (NullPointerException e) {
			setSize(LARGURA_PADRAO, ALTURA_PADRAO);
		}		
	}
	
	public void init() {
		String largura = getParameter("largura");
		String altura = getParameter("altura");
		redimensionar(largura, altura);
		Thread thread = new Thread(this);
		thread.start();
	}

	public void finalizar() {
		
	}
		
	public URL getURL(String caminho) {
		try {
			return new URL(getCodeBase(), caminho);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		return null;
	}
			
	public void carregarImagens(CarregamentoIT carregamento, int nArquivosMidia, int nArquivosCarregados) {
		figura.carregarImagens(carregamento, nArquivosMidia, nArquivosCarregados);
	}
	
	public void carregarSons(CarregamentoIT carregamento, int nArquivosMidia, int nArquivosCarregados) {
		audio.carregar(carregamento, nArquivosMidia, nArquivosCarregados);
	}
	
	public InputStream getInputStream(String caminho) throws IOException {
		return getURL(caminho).openStream();
	}
	
	public String getCaminho(String caminho) {
		return getURL(caminho).getPath();
	}
			
	public void playAudioClip(String caminho) {
		audio.playAudioClip(caminho);
	}
	
	public Image getImage(String caminho) {		
		return getImage(getCodeBase(), caminho);
	}
	
	public void desenhar(String caminho) {
		Desenho desenho = new DesenhoConcreto(this, caminho);
		getPainelPersonalizado().desenhar(desenho);
	}
	
	public Container getContainer() {
		return this;
	}
	
	public Figura getFigura() {
		return figura;
	}
	
	public Audio getSom() {
		return audio;
	}
	
	public PainelPersonalizado getPainelPersonalizado() {
		return painelPersonalizado;
	}

	public Desenho getFundo() {
		return fundo;
	}

}
