package italo.comecome;


import italo.comecome.audio.Audio;
import italo.comecome.conf.Conf;
import italo.comecome.entrada.Entrada;
import italo.comecome.gui.ContainerJogo;
import italo.comecome.gui.PainelPersonalizado;
import italo.comecome.gui.desenho.CarregamentoIT;
import italo.comecome.gui.desenho.Desenho;
import italo.comecome.gui.desenho.DesenhoCarregamento;
import italo.comecome.gui.desenho.DesenhoConcreto;
import italo.comecome.gui.figura.Figura;
import italo.comecome.logica.Jogo;

import java.awt.Color;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JFrame;

public class JogoFrame extends JFrame implements ContainerJogo {
	
	private static final long serialVersionUID = 1L;
	
	public final static String CAMINHO_IMAGEM_ABERTURA = "img/abertura/1.gif";
	public final static Color COR_FUNDO = Color.BLACK;
	public static boolean UNDECORATED = true;
	
	private Desenho fundo = null;	
	private PainelPersonalizado painelPersonalizado = null;
	
	private Audio audio;
	private Figura figura;
	
	public static void main(String[] args) {
		JogoFrame jogoFrame = new JogoFrame();
		jogoFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		jogoFrame.setExtendedState(JFrame.MAXIMIZED_BOTH);
		jogoFrame.setUndecorated(UNDECORATED);
		jogoFrame.setBackground(COR_FUNDO);		
		jogoFrame.setSize(Toolkit.getDefaultToolkit().getScreenSize());
		jogoFrame.setVisible(true);
		
		Figura figura = jogoFrame.getFigura();
		Audio audio = jogoFrame.getSom();
		
		Jogo jogo = new Jogo();
		
		Entrada entrada = new Entrada(jogoFrame);	
		HandlerJogo handlerJogo = new HandlerJogo(jogoFrame, entrada);
		HandlerEntradaJanela handlerEntradaJanela = new HandlerEntradaJanela(jogoFrame, jogo);
					
		Conf.getInstance().configurarJogo(jogo, figura, audio, jogoFrame.getWidth(), jogoFrame.getHeight());
		DesenhoCarregamento desenhoCarregamento = new DesenhoCarregamento(jogoFrame);
		desenhoCarregamento.carregar();
						
		jogo.addJogoListener(handlerJogo);
		entrada.addEntradaListener();
		entrada.addTecladoListener(handlerEntradaJanela);
								
		jogo.iniciar();		
	}
		
	public JogoFrame() {			
		fundo = new DesenhoConcreto(this, CAMINHO_IMAGEM_ABERTURA);		
		painelPersonalizado = new PainelPersonalizado(fundo);
		painelPersonalizado.setOpaque(false);
		setLayout(new GridLayout());	
		setContentPane(painelPersonalizado);
		audio = new Audio(this);
		figura = new Figura(this);
	}
	
	public void finalizar() {
		dispose();
		System.exit(0);
	}
	
	public URL getURL(String caminho) {
		try {
			File file = new File(caminho);
			return file.toURI().toURL();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		return null;
	}
			
	public void carregarImagens(CarregamentoIT carregamento, int nArquivosMidia, int nArquivosCarregados) {
		figura.carregarImagens(carregamento, nArquivosMidia, nArquivosCarregados);
	}
	
	public void carregarSons(CarregamentoIT carregamento, int nArquivosMidia, int nArquivosCarregados) {
		audio.carregar(carregamento, nArquivosMidia, nArquivosCarregados);
	}
	
	public InputStream getInputStream(String caminho) throws IOException {
		return getURL(caminho).openStream();
	}
	
	public String getCaminho(String caminho) {
		return getURL(caminho).getPath();
	}
			
	public void playAudioClip(String caminho) {
		audio.playAudioClip(caminho);
	}
	
	public Image getImage(String caminho) {		
		return Toolkit.getDefaultToolkit().getImage(caminho);
	}
			
	public void desenhar(String caminho) {
		Desenho desenho = new DesenhoConcreto(this, caminho);
		getPainelPersonalizado().desenhar(desenho);
	}
	
	public PainelPersonalizado getPainelPersonalizado() {
		return painelPersonalizado;
	}

	public Desenho getFundo() {
		return fundo;
	}

	public Container getContainer() {
		return this;
	}
	
	public Figura getFigura() {
		return figura;
	}
	
	public Audio getSom() {
		return audio;
	}

}
