package italo.comecome.gui.desenho;

import italo.comecome.gui.ContainerJogo;
import italo.comecome.gui.PainelPersonalizado;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

public class DesenhoCarregamento implements Desenho, CarregamentoIT {

	private static final long serialVersionUID = 1L;
	
	private final static double FATOR_LARGURA = 0.75;
	private final static double FATOR_ALTURA = 0.05;
	private final static Color COR_BARRA_PROGRESSO = Color.WHITE;
	private final static Color COR_PORCENTAGEM = Color.GREEN;
			
	private ContainerJogo container;
	private PainelPersonalizado painelPersonalizado;	
	private int porcentagem;
	private int nArquivosMidia;
	private int nArquivosCarregados;
	private int largura;
	private int altura;
	private int x;
	private int y;
	private int larguraBP;
	private int alturaBP;
	private int xBP;
	private int yBP;
	
	public DesenhoCarregamento(ContainerJogo container) {
		this.container = container;
		this.painelPersonalizado = container.getPainelPersonalizado();
		this.inicializar();
	}
		
	public void inicializar() {
		Dimension size = container.getContainer().getSize();
		int numeroDeFiguras = container.getFigura().getCaminhos().length;
		int numeroDeSons = container.getSom().getCaminhos().length;
		this.altura = size.height;
		this.largura = altura*4/3;
		this.x = (size.width-largura)/2;
		this.y = (size.height-altura)/2;
		this.larguraBP = (((int)(largura*FATOR_LARGURA))/100)*100;
		this.alturaBP = (int)(altura*FATOR_ALTURA);		
		this.xBP = x+((largura-larguraBP)/2);
		this.yBP = y+((altura-alturaBP)/2);		
		this.nArquivosMidia = (numeroDeSons+numeroDeFiguras);
		this.nArquivosCarregados = 0;
	}		

		
	public void carregar() {		
		container.carregarImagens(this, nArquivosMidia, nArquivosCarregados);
		container.carregarSons(this, nArquivosMidia, nArquivosCarregados);
	}
		
	public void carregado(int nArquivosCarregados) {		
		this.nArquivosCarregados = nArquivosCarregados;
		this.porcentagem = nArquivosCarregados*100/nArquivosMidia;
		this.painelPersonalizado.desenhar(this);	
	}
	
	public void desenhar(Graphics g) { 
		container.getFundo().desenhar(g);
		int larguraPreenchimento = (int)(porcentagem*(larguraBP/100));		
		g.setColor(COR_BARRA_PROGRESSO);
		g.drawRect(xBP, yBP, larguraBP, alturaBP);
		g.fillRect(xBP, yBP, larguraPreenchimento, alturaBP); 
		g.setColor(COR_PORCENTAGEM);
		g.drawString(String.valueOf(porcentagem)+"%", 
				(int)(xBP+(larguraBP*0.48)), (int)(yBP+(alturaBP*0.75))); 
	}

}
