package italo.comecome.logica;

import italo.comecome.personagem.ComeCome;
import italo.comecome.personagem.Monstrinho;
import italo.comecome.personagem.Personagem;
import italo.comecome.personagem.Posicao;
import italo.comecome.utilitarios.Utilitarios;

import java.util.ArrayList;
import java.util.List;

public class Fase implements FaseIT{
		
	private int numero;
	private Tabuleiro tabuleiro;
	private String caminhoImagemTabuleiro;
		
	private boolean[][] horizontalLivre;	
	private boolean[][] verticalLivre;		
	private boolean[][] bolinhasIniciais;
	private int[][] bolinhasGrandesIniciais;
	private int[][] direcoesIniciaisMonstrinhos;	
	private int[][] posicaoCelulaInicialComecome;	
	private int[][] posicoesCelulasIniciaisMonstrinhos;	
	private int[][] indicesPortas;
	private int nBolinhasIniciais;
	private int nBolinhasGrandesIniciais;
	
	private int nBolinhas;
	private int nBolinhasGrandes;
	private boolean[][] bolinhas;			
	private int[][] bolinhasGrandes;
	private Personagem comecome;
	private Personagem[] monstrinhos;
	private boolean executando = false;

	private List<FaseListener> listeners = new ArrayList<FaseListener>();
	
	public Fase(Personagem comecome, Personagem[] monstrinhos, Tabuleiro tabuleiro, String caminhoImagemTabuleiro, int numero) {
		this.numero = numero;
		this.comecome = comecome;
		this.monstrinhos = monstrinhos;
		this.tabuleiro = tabuleiro;
		this.caminhoImagemTabuleiro = caminhoImagemTabuleiro;
	}
				
	public Fase(Tabuleiro tabuleiro) {
		this.tabuleiro = tabuleiro;
	}
	
	public Fase() {}
	
	public void inicializar() {	
		this.bolinhas = Utilitarios.copia(bolinhasIniciais);
		this.bolinhasGrandes = Utilitarios.copia(bolinhasGrandesIniciais);
		this.nBolinhas = nBolinhasIniciais;
		this.nBolinhasGrandes = nBolinhasGrandesIniciais;
		this.executando = false;
	}
			
	public void addFaseListener(FaseListener l) {
		listeners.add(l);
	}
	
	public void removeFaseListener(FaseListener l) {
		listeners.remove(l);
	}
	
	public void executar() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).executar(this);
	}
	
	public void iniciar() {
		inicializar();
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).iniciar(this);
	}
	
	public void reiniciar() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).reiniciar(this);
	}
		
	public void finalizar() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).finalizar(this);
	}
	
	public void sairDaCasaInicio(Monstrinho monstrinho) {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).sairDaCasaInicio(this, monstrinho);
	}
	
	public void entrarNaCasa(Monstrinho monstrinho) {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).entrarNaCasa(this, monstrinho);
	}
	
	public void sairDaCasa(Monstrinho monstrinho) {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).sairDaCasa(this, monstrinho);
	}
	
	public void mover(Personagem personagem) {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).mover(this, personagem);
	}
	
	public void mostrarCaminhoOtimo(Monstrinho monstrinho, boolean b) {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).mostrarCaminhoOtimo(this, monstrinho, b);
	}
	
	public void comeuBolinha() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).comeuBolinha(this);
	}
	
	public void comeuBolinhaGrande() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).comeuBolinhaGrande(this);
	}

	public void comeuMonstrinho() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).comeuMonstrinho(this);
	}
	
	private boolean getHorizontalLivreValor(int y, int x, int direcao, boolean atravessarParedes) {		
		if(y >= 0 && y < horizontalLivre.length && x >= 0 && x < horizontalLivre[0].length){
			if(direcao == Personagem.CIMA) {
				if(atravessarParedes) {
					return (y != 0);
				} else {
					return horizontalLivre[y][x];
				}				
			} else if(direcao == Personagem.BAIXO && y+1 < horizontalLivre.length && x < horizontalLivre[0].length){
				if(atravessarParedes) {
					return (y+1 != (horizontalLivre.length-1));
				} else {
					return horizontalLivre[y+1][x];
				}
			}
		}		
		return false;		
	}
	
	private boolean getVerticalLivreValor(int y, int x, int direcao, boolean atravessarParedes) {
		if(y >= 0 && y < verticalLivre.length && x >= 0 && x < verticalLivre[0].length){
			if(direcao == Personagem.TRAZ) {
				if(atravessarParedes) {
					return ((x != 0) || (y == verticalLivre.length/2));
				} else {
					return verticalLivre[y][x];
				}
			} else if(direcao == Personagem.FRENTE && y < verticalLivre.length && x+1 < verticalLivre[0].length){
				if(atravessarParedes) {
					return ((x+1 != (verticalLivre[0].length-1)) || (y == verticalLivre.length/2));
				} else {
					return verticalLivre[y][x+1];
				}
			}
		}
		return false;		
	}
			
	public boolean isCelulaLivre(int y, int x, int direcao, boolean atravessarParedes) {
		boolean livre = false;
		if(direcao == Personagem.BAIXO || direcao == Personagem.CIMA) {
			livre = getHorizontalLivreValor(y, x, direcao, atravessarParedes);
		} else {
			livre = getVerticalLivreValor(y, x, direcao, atravessarParedes);			
		}		
		return livre;
	}
		
	public boolean isPosicaoCelula(int x, int y) {
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		return (x % larguraCelula == 0 && y % alturaCelula == 0);
	}
	
	public boolean isPosicaoCelula(Posicao p) {
		return isPosicaoCelula(p.getX(), p.getY());
	}
	
	public boolean podeMover(Personagem personagem) {
		boolean atravessarParedes = 
			(personagem instanceof ComeCome ? ((ComeCome)personagem).isAtravessarParedes() : false);
		Posicao posicao = personagem.getPosicao();
		int x = posicao.getX();
		int y = posicao.getY();
		int direcao = personagem.getDirecao();
		int novaDirecao = personagem.getNovaDirecao();
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		boolean podeMover = true;
		if(isPosicaoCelula(x, y)) {		
			if(direcao == novaDirecao) {
				podeMover = isCelulaLivre(y/alturaCelula, x/larguraCelula, direcao, atravessarParedes);									
			} else {
				boolean novaDirLivre = isCelulaLivre(y/alturaCelula, x/larguraCelula, novaDirecao, atravessarParedes);
				if(novaDirLivre) {
					podeMover = true;				
				} else {
					podeMover = isCelulaLivre(y/alturaCelula, x/larguraCelula, direcao, atravessarParedes);									
				}
			}
		}
		return podeMover;		
	}
	
	public boolean podeMoverParaNovaDirecao(Personagem personagem) {
		boolean atravessarParedes = 
			(personagem instanceof ComeCome ? ((ComeCome)personagem).isAtravessarParedes() : false);
		Posicao posicao = personagem.getPosicao();
		int x = posicao.getX();
		int y = posicao.getY();
		int novaDirecao = personagem.getNovaDirecao();
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		if(isPosicaoCelula(x, y))  
			return isCelulaLivre(y/alturaCelula, x/larguraCelula, novaDirecao, atravessarParedes);		
		return false;
	}
					
	public boolean podeEntrarNaCasa(int xH, int yH) {
		for(int i = 0; i < indicesPortas.length; i++) {
			int xP = indicesPortas[i][1];
			int yP = indicesPortas[i][0];
			if(xH == xP && yH+1 == yP)
				return true;
		}	
		return false;
	}
	
	public boolean podeSairDaCasa(int xH, int yH) {
		for(int i = 0; i < indicesPortas.length; i++) {
			int xP = indicesPortas[i][1];
			int yP = indicesPortas[i][0];
			if(xH == xP && yH == yP)
				return true;
		}		
		return false;
	}

	public Posicao criarPosicaoNova(int direcao, Posicao pAtual) {
		int larguraTabuleiro = tabuleiro.getLarguraTabuleiro();
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		int larguraMiniCelula = tabuleiro.getLarguraMiniCelula();
		int alturaMiniCelula = tabuleiro.getAlturaMiniCelula();
		int nCelulasVertical = tabuleiro.getNCelulasVertical();		
		int pXMin = 0;
		int pXMax = larguraTabuleiro-larguraCelula; 
		int yMeio = alturaCelula*(nCelulasVertical/2);
		int x = pAtual.getX();
		int y = pAtual.getY();
		switch(direcao) {
			case Personagem.TRAZ:
				if(y != yMeio || x >= pXMin) {
					x -= larguraMiniCelula;										
				} else {
					x = pXMax;
				}
				break;
			case Personagem.FRENTE:
				if(y != yMeio || x <= pXMax) {
					x += larguraMiniCelula;										
				} else {
					x = pXMin;
				}
				break;			
			case Personagem.BAIXO:
				y += alturaMiniCelula;
				break;
			case Personagem.CIMA:
				y -= alturaMiniCelula;
				break;
			
		}	
		return new Posicao(x, y);
	}
			
	public int getDistanciaLimite(int dificultade) {
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		return dificultade*Math.max(larguraCelula, alturaCelula);
	}
	
	public Personagem[] getMonstrinhosProximos(ComeCome comecome, double distancia) {
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		int nMonstrinhosProx = getNMonstrinhosProximos(comecome, distancia);
		Personagem[] monstrinhosProx = new Personagem[nMonstrinhosProx];
		int xCC = comecome.getPosicao().getX()+(larguraCelula/2);
		int yCC = comecome.getPosicao().getY()+(alturaCelula/2);
		int cont = 0;
		for(int i = 0; i < monstrinhos.length; i++) {
			Monstrinho monstrinho = (Monstrinho)monstrinhos[i];
			int xM = monstrinho.getPosicao().getX()+(larguraCelula/2);
			int yM = monstrinho.getPosicao().getY()+(alturaCelula/2);
			double dist = Utilitarios.distancia(xCC, yCC, xM, yM);
			if(dist < distancia) {
				monstrinhosProx[cont] = monstrinho;
				cont++;
			}
		}	
		return (monstrinhosProx);
	}	
	
	public int getNMonstrinhosProximos(ComeCome comecome, double distancia) {
		int larguraCelula = tabuleiro.getLarguraCelula();
		int alturaCelula = tabuleiro.getAlturaCelula();
		int xCC = comecome.getPosicao().getX()+(larguraCelula/2);
		int yCC = comecome.getPosicao().getY()+(alturaCelula/2);
		int cont = 0;
		for(int i = 0; i < monstrinhos.length; i++) {
			Monstrinho monstrinho = (Monstrinho)monstrinhos[i];
			int xM = monstrinho.getPosicao().getX()+(larguraCelula/2);
			int yM = monstrinho.getPosicao().getY()+(alturaCelula/2);
			double dist = Utilitarios.distancia(xCC, yCC, xM, yM);
			if(dist < distancia)
				cont++;			
		}
		return cont;
	}
	
	public boolean ehBolinhaGrande(int yBolinha, int xBolinha) {
		for(int i = 0; i < bolinhasGrandes.length; i++)
			if(yBolinha == bolinhasGrandes[i][0] && xBolinha == bolinhasGrandes[i][1])
				return (bolinhasGrandes[i][0] != -1 && bolinhasGrandes[i][1] != -1);									
		return false;
	}
	
	public void deletaBolinhaGrande(int yBolinha, int xBolinha) {
		boolean ehBolinhaGrande = false;
		for(int i = 0; i < bolinhasGrandes.length && !ehBolinhaGrande; i++){
			if(yBolinha == bolinhasGrandes[i][0] && xBolinha == bolinhasGrandes[i][1]) {
				bolinhasGrandes[i][0] = -1;
				bolinhasGrandes[i][1] = -1;
				ehBolinhaGrande = true;							
			}
		}
	}
	
	public void decrementarBolinhas() {
		nBolinhas--;
	}
	
	public void decrementarBolinhasGrandes() {
		nBolinhasGrandes--;
	}
		
	public int getIndiceMonstrinho(Personagem p) {
		for(int i = 0; i < monstrinhos.length; i++)
			if(p.equals(monstrinhos[i]))
				return i;		
		return -1;
	}
	
	public void finalizarMonstrinhos() {
		for(int i = 0; i < monstrinhos.length; i++)
			((Monstrinho)monstrinhos[i]).setFim(true);
	}	
	
	public void comerBolinha(int xBolinha, int yBolinha) {
		if(bolinhas != null && xBolinha >= 0 && xBolinha < bolinhas[0].length 
				&& yBolinha >= 0 && yBolinha < bolinhas.length) {			
			if(bolinhas[yBolinha][xBolinha]) {
				bolinhas[yBolinha][xBolinha] = false;				
				decrementarBolinhas();
				comeuBolinha();
			}
			if(ehBolinhaGrande(yBolinha, xBolinha)) {	
				deletaBolinhaGrande(yBolinha, xBolinha);
				decrementarBolinhasGrandes();
				comeuBolinhaGrande();				
			}
		}
	}

	public boolean[][] getHorizontalLivre() {
		return horizontalLivre;
	}

	public void setHorizontalLivre(boolean[][] horizontalLivre) {
		this.horizontalLivre = horizontalLivre;
	}

	public boolean[][] getVerticalLivre() {
		return verticalLivre;
	}

	public void setVerticalLivre(boolean[][] verticalLivre) {
		this.verticalLivre = verticalLivre;
	}

	public String getCaminhoImagemTabuleiro() {
		return caminhoImagemTabuleiro;
	}

	public void setCaminhoImagemTabuleiro(String caminhoImagemTabuleiro) {
		this.caminhoImagemTabuleiro = caminhoImagemTabuleiro;
	}

	public boolean[][] getBolinhas() {
		return bolinhas;
	}

	public int getNBolinhas() {
		return nBolinhas;
	}
	
	public int getNBolinhasGrandes() {
		return nBolinhasGrandes;
	}
	
	public int[][] getDirecoesIniciaisMonstrinhos() {
		return direcoesIniciaisMonstrinhos;
	}

	public void setDirecoesIniciaisMonstrinhos(int[][] direcoesIniciaisMonstrinhos) {
		this.direcoesIniciaisMonstrinhos = direcoesIniciaisMonstrinhos;
	}
	
	public int[][] getPosicoesIniciaisComecome() {
		return posicaoCelulaInicialComecome;
	}

	public void setPosicoesIniciaisComecome(int[][] posicoesIniciaisComecome) {
		this.posicaoCelulaInicialComecome = posicoesIniciaisComecome;
	}
	
	public int[][] getPosicoesIniciaisMonstrinhos() {
		return posicoesCelulasIniciaisMonstrinhos;
	}

	public void setPosicoesIniciaisMonstrinhos(int[][] posicoesIniciaisMonstrinhos) {
		this.posicoesCelulasIniciaisMonstrinhos = posicoesIniciaisMonstrinhos;
	}

	public ComeCome getComeCome() {
		return (ComeCome)comecome;
	}
	
	public void setComeCome(ComeCome comecome) {
		this.comecome = comecome;
	}
	
	public Personagem[] getMonstrinhos() {
		return monstrinhos;
	}

	public void setMonstrinhos(Personagem[] monstrinhos) {
		this.monstrinhos = monstrinhos;
	}

	public int[][] getIndicesPortas() {
		return indicesPortas;
	}

	public void setIndicesPortas(int[][] indicesPortas) {
		this.indicesPortas = indicesPortas;
	}

	public int[][] getBolinhasGrandes() {
		return bolinhasGrandes;
	}

	public void setBolinhasGrandes(int[][] bolinhasGrandes) {
		this.bolinhasGrandes = bolinhasGrandes;
	}

	public int getNumero() {
		return numero;
	}

	public void setNumero(int numero) {
		this.numero = numero;
	}

	public int[][] getPosicaoCelulaInicialComecome() {
		return posicaoCelulaInicialComecome;
	}

	public void setPosicaoCelulaInicialComecome(int[][] posicaoCelulaInicialComecome) {
		this.posicaoCelulaInicialComecome = posicaoCelulaInicialComecome;
	}

	public int[][] getPosicoesCelulasIniciaisMonstrinhos() {
		return posicoesCelulasIniciaisMonstrinhos;
	}

	public void setPosicoesCelulasIniciaisMonstrinhos(
			int[][] posicoesCelulasIniciaisMonstrinhos) {
		this.posicoesCelulasIniciaisMonstrinhos = posicoesCelulasIniciaisMonstrinhos;
	}

	public boolean[][] getBolinhasIniciais() {
		return bolinhasIniciais;
	}

	public void setBolinhasIniciais(boolean[][] bolinhasIniciais) {
		this.bolinhasIniciais = bolinhasIniciais;
	}

	public int[][] getBolinhasGrandesIniciais() {
		return bolinhasGrandesIniciais;
	}

	public void setBolinhasGrandesIniciais(int[][] bolinhasGrandesIniciais) {
		this.bolinhasGrandesIniciais = bolinhasGrandesIniciais;
	}

	public int getNBolinhasIniciais() {
		return nBolinhasIniciais;
	}

	public void setNBolinhasIniciais(int bolinhasIniciais) {
		nBolinhasIniciais = bolinhasIniciais;
	}

	public int getNBolinhasGrandesIniciais() {
		return nBolinhasGrandesIniciais;
	}

	public void setNBolinhasGrandesIniciais(int bolinhasGrandesIniciais) {
		nBolinhasGrandesIniciais = bolinhasGrandesIniciais;
	}

	public boolean isExecutando() {
		return executando;
	}

	public void setExecutando(boolean executando) {
		this.executando = executando;
	}

	public Tabuleiro getTabuleiro() {
		return tabuleiro;
	}

	public void setTabuleiro(Tabuleiro tabuleiro) {
		this.tabuleiro = tabuleiro;
	}

}
