package italo.comecome.logica;

import italo.comecome.truque.Truque;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Jogo implements JogoIT{
	
	public final static char[] TRUQUE_ATRAVESSAR_PAREDES = new char[]{'a','t','r','a','v','e','s','s','a','r',' ','p','a','r','e','d','e','s'};
	public final static char[] TRUQUE_MUDAR_DE_FASE = new char[]{'f','a','s','e','?'};	
	public final static char[] TRUQUE_MUDAR_NUMERO_DE_VIDAS = new char[]{'v','i','d','a','s', Truque.CORINGA, Truque.CORINGA, Truque.CORINGA};  

	public final static int DIFICULDADE_FACIL = 1;
	public final static int DIFICULDADE_NORMAL = 2;
	public final static int DIFICULDADE_DIFICIL = 3;
	public final static int DIFICULDADE_SUPER_DIFICIL = 4;
	
	public final static int CHAVE_TRUQUE_ATRAVESSAR_PAREDES = 1;
	public final static int CHAVE_TRUQUE_MUDAR_DE_FASE = 2;	
	public final static int CHAVE_TRUQUE_MUDAR_NUMERO_DE_VIDAS = 2;

	private Fase[] fases = null;	
	private int indiceFaseAtual = 0;
	private int nVidasConf = 0;
	private int nVidas = 0;
	private int nVidasAtual = 0;
	private int dificuldade = DIFICULDADE_NORMAL;
	private boolean atravessarParedes = false;
	private Map<Integer, Truque> truquesJogo = new HashMap<Integer, Truque>(); 
	private Map<Integer, Truque> truquesMenuJogo = new HashMap<Integer, Truque>();
	private Map<Integer, Truque> truquesMenuOpcoes = new HashMap<Integer, Truque>();
	private Map<Integer, Truque> truquesJanela = new HashMap<Integer, Truque>(); 
	
	private List<JogoListener> listeners = new ArrayList<JogoListener>();
	
	public Jogo() {
		truquesJogo.put(CHAVE_TRUQUE_ATRAVESSAR_PAREDES, new Truque(TRUQUE_ATRAVESSAR_PAREDES));
		truquesJogo.put(CHAVE_TRUQUE_MUDAR_DE_FASE, new Truque(TRUQUE_MUDAR_DE_FASE));
		truquesMenuJogo.put(CHAVE_TRUQUE_MUDAR_NUMERO_DE_VIDAS, new Truque(TRUQUE_MUDAR_NUMERO_DE_VIDAS));
	}

	public void inicializar() {
		indiceFaseAtual = 0;
		nVidasAtual = nVidas;		
	}
			
	public void executar() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).executar(this);
	}
	
	public void iniciar() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).iniciar(this);
	}
		
	public void startar() {
		inicializar();
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).startar(this);
	}
	
	public void sair() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).sair(this);
	}
	
	public void finalizar() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).finalizar(this);
	}
	
	public void pararExecucao() {
		for(int i = 0; i < listeners.size(); i++){					
			listeners.get(i).pararExecucao(this);
		}
	}
	
	public void continuarExecucao() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).continuarExecucao(this);
	}
	
	public void mudarDeFase(int nFase) {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).mudarDeFase(this, nFase);
	}
	
	public void zerou() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).zerou(this);
	}
	
	public void gameover() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).gameover(this);		
	}
	
	public void perdeu() {		
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).perdeu(this);		
	}

	public void passou() {
		for(int i = 0; i < listeners.size(); i++)
			listeners.get(i).passou(this);
	}
			
	public boolean isFimJogo() {
		return (nVidasAtual <= 0 || indiceFaseAtual >= fases.length);
	}
			
	public Fase getFaseAtual() {
		if(indiceFaseAtual < fases.length)
			return fases[indiceFaseAtual];
		return fases[fases.length-1];
	}

	public Fase[] getFases() {
		return fases;
	}

	public void setFases(Fase[] fases) {
		this.fases = fases;
	}
	
	public int getDificuldade() {
		return dificuldade;
	}

	public void setDificuldade(int dificuldade) {
		this.dificuldade = dificuldade;
	}

	public int getIndiceFaseAtual() {
		return indiceFaseAtual;
	}

	public void setIndiceFaseAtual(int indiceFaseAtual) {
		this.indiceFaseAtual = indiceFaseAtual;
	}
	
	public void addJogoListener(JogoListener listener) {
		listeners.add(listener);
	}
	
	public void removeJogoListener(JogoListener listener) {
		listeners.remove(listener);
	}

	public int getNVidas() {
		return nVidas;
	}

	public void setNVidas(int nVidas) {
		this.nVidas = nVidas;
		this.nVidasAtual = nVidas;
	}
	
	public int getNVidasAtual() {
		return nVidasAtual;
	}

	public void setNVidasAtual(int nVidas) {
		nVidasAtual = nVidas;
	}

	public boolean isAtravessarParedes() {
		return atravessarParedes;
	}

	public void setAtravessarParedes(boolean atravessarParedes) {
		this.atravessarParedes = atravessarParedes;
	}

	public Map<Integer, Truque> getTruquesJogo() {
		return truquesJogo;
	}

	public Map<Integer, Truque> getTruquesMenuJogo() {
		return truquesMenuJogo;
	}

	public Map<Integer, Truque> getTruquesMenuOpcoes() {
		return truquesMenuOpcoes;
	}

	public Map<Integer, Truque> getTruquesJanela() {
		return truquesJanela;
	}

	public int getNVidasConf() {
		return nVidasConf;
	}

	public void setNVidasConf(int vidasConf) {
		nVidasConf = vidasConf;
	}

}
