package italo.comecome.personagem.util;

import italo.comecome.a_estrela.AEstrela;
import italo.comecome.a_estrela.CaminhoAEstrela;
import italo.comecome.a_estrela.CelulaAEstrela;
import italo.comecome.logica.Fase;
import italo.comecome.personagem.ComeCome;
import italo.comecome.personagem.Monstrinho;
import italo.comecome.personagem.Personagem;
import italo.comecome.utilitarios.Utilitarios;

import java.util.List;

public class PersonagemUtil {
	
	public static int getDirecaoAleatoria() {
		return Utilitarios.random(Personagem.DIRECAO_MINIMA, Personagem.DIRECAO_MAXIMA);
	}
	
	public static int getDirecaoOposta(int direcao) {
		switch(direcao) {
			case Personagem.FRENTE:	return Personagem.TRAZ;
			case Personagem.TRAZ:   return Personagem.FRENTE;
			case Personagem.BAIXO:  return Personagem.CIMA;
			case Personagem.CIMA:   return Personagem.BAIXO;
			default:  return -1;
		}
	}
	
	public static String getDirecao(int direcao) {
		switch(direcao) {
			case Personagem.FRENTE:	return "frente";
			case Personagem.TRAZ:   return "traz";
			case Personagem.BAIXO:  return "baixo";
			case Personagem.CIMA:   return "cima";
			default:  return "";
		}
	}
	
	public static int getDirecaoINT(String direcao) {
		if(direcao.equalsIgnoreCase("frente")) {
			return Personagem.FRENTE;
		} else if(direcao.equalsIgnoreCase("baixo")) {
			return Personagem.BAIXO;
		} else if(direcao.equalsIgnoreCase("traz")) {
			return Personagem.TRAZ;
		} else if(direcao.equalsIgnoreCase("cima")) {
			return Personagem.CIMA;
		}
		return -1;
	}
			
	public static String getEstado(int estado) {
		switch(estado) {
			case Monstrinho.FORTE:       return "forte";
			case Monstrinho.MEIO_FRACO:  return "meiofraco";	
			case Monstrinho.FRACO:       return "fraco";	
			case Monstrinho.MUITO_FRACO: return "muitofraco";	
			default: return "";
		}		
	}
	
	public static int getDirecao(String direcao) {		
		if(direcao.equals("frente")) {
			return Personagem.FRENTE;
		} else if(direcao.equals("traz")) {
			return Personagem.TRAZ;
		} else if(direcao.equals("baixo")) {
			return Personagem.BAIXO;
		} else if(direcao.equals("cima")) {
			return Personagem.CIMA;
		}
		return -1;
	}
	
	public static double distancia(Personagem p1, Personagem p2) {
		int xP1 = p1.getPosicao().getX();
		int yP1 = p1.getPosicao().getY();
		int xP2 = p2.getPosicao().getX();
		int yP2 = p2.getPosicao().getY();
		return Utilitarios.distancia(xP1, yP1, xP2, yP2);		
	}
	
	public static int getDirecaoMelhor(Personagem p1, Personagem p2, int larguraCelula, int alturaCelula, Melhor melhor) {
		int x1 = p1.getPosicao().getX()+(larguraCelula/2);
		int y1 = p1.getPosicao().getY()+(alturaCelula/2);
		int x2 = p2.getPosicao().getX()+(larguraCelula/2);
		int y2 = p2.getPosicao().getY()+(alturaCelula/2);
		int[] direcoes = {Monstrinho.FRENTE, Monstrinho.BAIXO, Monstrinho.TRAZ, Monstrinho.CIMA};
		int[] posicoesIncX = {larguraCelula/3, 0, -alturaCelula/3, 0};
		int[] posicoesIncY = {0, alturaCelula/3, 0, -alturaCelula/3};
		int melhorDir = direcoes[0];
		for(int i = 1; i < direcoes.length; i++) {
			if(direcoes[i] == p1.getDirecao()) 
				continue;
			double dist = Utilitarios.distancia(x1+posicoesIncX[i], y1+posicoesIncY[i], x2, y2);
			double menorDist = Utilitarios.distancia(x1+posicoesIncX[0], y1+posicoesIncY[0], x2, y2);
			if(melhor.comparacao(dist, menorDist)) { 
				menorDist = dist;
				melhorDir = direcoes[i];
			}
		}	
		return melhorDir;
	}
		
	public static int getDirecaoMaisProxima(Monstrinho monstrinho, ComeCome comecome, Fase fase) {
		int larguraCelula = fase.getTabuleiro().getLarguraCelula();
		int alturaCelula = fase.getTabuleiro().getAlturaCelula();
		int x1 = monstrinho.getPosicao().getX()/larguraCelula;
		int y1 = monstrinho.getPosicao().getY()/alturaCelula;
		int x2 = comecome.getPosicao().getX()/larguraCelula;
		int y2 = comecome.getPosicao().getY()/alturaCelula;
		CelulaAEstrela cInicial = new CelulaAEstrela(x1, y1);
		CelulaAEstrela cAlvo = new CelulaAEstrela(x2, y2); 
		AEstrela aestrela = new AEstrela(fase, cInicial, cAlvo);
		CaminhoAEstrela caminhoAEstrela = aestrela.executar();			
		if(caminhoAEstrela != null) { 
			List<CelulaAEstrela> caminho = caminhoAEstrela.getCaminho();			
			monstrinho.setCaminhoAEstrela(caminho);
			int melhorDir = Personagem.FRENTE;
			CelulaAEstrela celula = caminho.get(caminho.size()-1);
			int[][] vizinhas = {
				{ -1,  0, Personagem.TRAZ   }, 
				{ 0 ,  1, Personagem.BAIXO  }, 
				{ 1 ,  0, Personagem.FRENTE }, 
				{ 0 , -1, Personagem.CIMA   }
			};			
			for(int i = 0; i < vizinhas.length; i++) {
				int x = x1+vizinhas[i][0];
				int y = y1+vizinhas[i][1]; 
				int direcao = vizinhas[i][2];
				if(x == celula.getX() && y == celula.getY()) {						
					melhorDir = direcao;
					break;
				}
			}
			return melhorDir;
		} else {
			return getDirecaoMelhor(monstrinho, comecome, larguraCelula, alturaCelula, new MaisProxima());
		}	
	}
	
}
