package italo.comecome.truque;

import java.util.ArrayList;
import java.util.List;

public class Truque implements TruqueListener {

	public final static char CORINGA = '?';
	
	public final static int TRUQUE_ATIVO = 1;
	public final static int TRUQUE_NAO_ATIVO = 2;
		
	private char[] truque;
	private char[] truqueDigitado;
	private int indice;
	private int status = TRUQUE_NAO_ATIVO;
	private List<TruqueListener> truqueListeners = new ArrayList<TruqueListener>();
	
	public Truque(char[] truque) {
		this.truque = truque;
		this.truqueDigitado = new char[truque.length];
		this.indice = 0;
	}
	
	public void teclaPressionada(char tecla) {		
		char teclaDigitada = Character.toLowerCase(tecla);
		char teclaTruque = Character.toLowerCase(truque[indice]);
		if(teclaDigitada == teclaTruque || teclaTruque == CORINGA) {
			truqueDigitado[indice] = teclaDigitada;			
			if(indice < truque.length-1) {				
				indice++;
			} else {
				indice = 0;
				status = (status == TRUQUE_ATIVO ? TRUQUE_NAO_ATIVO: TRUQUE_ATIVO);
				truqueCompleto(this);	
			}
		} else {
			indice = 0;
		}		
	}
	
	public void truqueCompleto(Truque truque) {
		List<TruqueListener> listeners = truque.getTruqueListeners();
		for(TruqueListener l:listeners)
			l.truqueCompleto(truque);
	}

	public void addTruqueListener(TruqueListener listener) {
		truqueListeners.add(listener);
	}
	
	public void removeTruqueListener(TruqueListener listener) {
		truqueListeners.remove(listener);
	}

	public char[] getTruque() {
		return truque;
	}

	public int getIndice() {
		return indice;
	}

	public List<TruqueListener> getTruqueListeners() {
		return truqueListeners;
	}

	public int getStatus() {
		return status;
	}

	public char[] getTruqueDigitado() {
		return truqueDigitado;
	}
	
}
