package italo.comecome.a_estrela;


import italo.comecome.a_estrela.util.UtilitariosAEstrela;
import italo.comecome.logica.Fase;
import italo.comecome.personagem.Personagem;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class AEstrela {
 
	private Fase fase;
	private CelulaAEstrela cInicial;
	private CelulaAEstrela cAlvo;
	
	private CelulaAEstrela[][] celulas;
	private List<PosicaoAEstrela> listaAberta = new LinkedList<PosicaoAEstrela>();
	private List<PosicaoAEstrela> listaFechada = new LinkedList<PosicaoAEstrela>();
	
	public AEstrela(Fase fase, CelulaAEstrela cInicial, CelulaAEstrela cAlvo) {
		this.fase = fase;
		this.cInicial = cInicial;
		this.cAlvo = cAlvo;		
		inicializar();
	}
			
	public CaminhoAEstrela executar() {
		CelulaAEstrela celula = null;
		listaAberta.add(cInicial.getPosicao()); 		
		boolean encontrouCaminho = false;
		while(!listaAberta.isEmpty() && !encontrouCaminho) {
			ordenaPorF(listaAberta);
			PosicaoAEstrela posicao = listaAberta.remove(0);			
			listaFechada.add(posicao);
			celula = celulas[posicao.getX()][posicao.getY()];			
			if(cAlvo.posicaoIgual(posicao)) {
				encontrouCaminho = true;
			} else {
				List<PosicaoAEstrela> vizinhas = getCelulasVizinhasSemObstaculo(posicao);
				ordenaPorF(vizinhas);
				for(PosicaoAEstrela p:vizinhas) {
					CelulaAEstrela cVizinha = celulas[p.getX()][p.getY()];
					if(estaNaListaFechada(cVizinha))
						continue;				
					int distAlvo = (int) UtilitariosAEstrela.distancia(cVizinha, cAlvo);
					int	gNovo = celula.getG()+10;
					int hNovo = distAlvo*10;
					if(!this.estaNaListaAberta(cVizinha)) {
						listaAberta.add(p);					
						cVizinha.setPai(posicao);
						cVizinha.setG(gNovo);
						cVizinha.setH(hNovo);
					} else if(gNovo < cVizinha.getG()) {
						cVizinha.setPai(posicao);
						cVizinha.setG(gNovo);
						cVizinha.setH(hNovo);						
					}
				}
			}
		}
		if(listaAberta.isEmpty())
			return null;
		CaminhoAEstrela caminhoAEstrela = new CaminhoAEstrela();
		List<CelulaAEstrela> caminho = new ArrayList<CelulaAEstrela>();
		while(!cInicial.posicaoIgual(celula.getPosicao())) {				
			caminho.add(celula);	
			celula = celulas[celula.getPai().getX()][celula.getPai().getY()];			
		}		
		caminhoAEstrela.setCaminho(caminho);
		return caminhoAEstrela;
	}
	
	private void inicializar() {
		int nCelulasH = fase.getTabuleiro().getNCelulasHorizontal();
		int nCelulasV = fase.getTabuleiro().getNCelulasVertical();
		celulas = new CelulaAEstrela[nCelulasH][nCelulasV];
		for(int i = 0; i < nCelulasH; i++) {
			for(int j = 0; j < nCelulasV; j++) 
				celulas[i][j] = new CelulaAEstrela(i, j);							
		}
	}
	
	private List<PosicaoAEstrela> getCelulasVizinhasSemObstaculo(PosicaoAEstrela posicao) {		
		List<PosicaoAEstrela> celulasP = new LinkedList<PosicaoAEstrela>();
		int nCelulasH = fase.getTabuleiro().getNCelulasHorizontal();
		int nCelulasV = fase.getTabuleiro().getNCelulasVertical();				
		int[][] vizinhas = {
				{ -1,  0, Personagem.TRAZ   }, 
				{ 0 ,  1, Personagem.BAIXO  }, 
				{ 1 ,  0, Personagem.FRENTE }, 
				{ 0 , -1, Personagem.CIMA   }
		};
		for(int i = 0; i < vizinhas.length; i++) {
			int xP = posicao.getX();
			int yP = posicao.getY();
			int x = xP+vizinhas[i][0];
			int y = yP+vizinhas[i][1]; 
			int direcao = vizinhas[i][2];
			if(x >= 0 && y >= 0 && x < nCelulasH && y < nCelulasV) {	
				if(fase.isCelulaLivre(yP, xP, direcao, false)) {
					celulasP.add(new PosicaoAEstrela(x, y)); 
				}
			}		
		}
		return celulasP;		
	}
	
	private boolean estaNaListaAberta(CelulaAEstrela celula) {
		for(PosicaoAEstrela p:listaAberta) 
			if(celula.posicaoIgual(p)) 
				return true;
		return false;
	}
	
	private boolean estaNaListaFechada(CelulaAEstrela celula) {
		for(PosicaoAEstrela p:listaFechada) 
			if(isPosicoesIguais(p, celula.getPosicao())) 
				return true;
		return false;
	}
		
	private boolean isPosicoesIguais(PosicaoAEstrela pC1, PosicaoAEstrela pC2) {
		if(pC1 != null && pC2 != null) {
			return (pC1.getX() == pC2.getX() && pC1.getY() == pC2.getY());
		} else if(pC1 == null && pC2 == null) {
			throw new RuntimeException("Erro: Posicao nula.");
		} else {
			return false;
		}		       
	}
	
	public void ordenaPorF(List<PosicaoAEstrela> posicoes) {
		boolean ordenado;
		do {
			ordenado = true;
			for(int i = 0; i < (posicoes.size()-1); i++) {
				PosicaoAEstrela p1 = posicoes.get(i);
				PosicaoAEstrela p2 = posicoes.get(i+1);
				CelulaAEstrela c1 = celulas[p1.getX()][p1.getY()];
				CelulaAEstrela c2 = celulas[p2.getX()][p2.getY()];
				if(c1.getF() > c2.getF()) {
					posicoes.set(i, p2);
					posicoes.set(i+1, p1);
					ordenado = false;
				}
			}
		} while(!ordenado);
	}
	
}
