package italo.comecome.personagem;

import italo.comecome.logica.Fase;
import italo.comecome.logica.Jogo;

public class ComeCome extends Personagem {
	
	private ComeComeIT comecomeIT = new ComeComeIMP(jogo);	
	private boolean parar = false;	
	private boolean mudarFase = false;
	
	public ComeCome(Jogo jogo, String nome, int id, int sleep) {
		super(jogo, nome, id, sleep);	
	}

	public void executar() {
		boolean ganhou = false;
		boolean perdeu = false;			
		while(!ganhou && !perdeu && !mudarFase) {		
			while(parar && !mudarFase) 
				esperar();
			perdeu = comecomeIT.monstrinhoPegou(this);
			if(!perdeu && !mudarFase) {
				comecomeIT.mover(this);
				comecomeIT.comer(this);
				if(!parar && !mudarFase)
					comecomeIT.dormir(this);
				ganhou = comecomeIT.ganhou(this);
			}								
		}
		if(!mudarFase)
			finalizar(ganhou);
	}
	
	public void iniciar() {
		setDirecao(FRENTE);
		setNovaDirecao(FRENTE);
		setParar(false);
		setMudarFase(false);		
	}
			
	public boolean podeMover() {		
		Fase fase = jogo.getFaseAtual();		
		if(fase == null) 
			return false;
		return fase.podeMover(this);					
	}
	
	private void finalizar(boolean ganhou) {
		jogo.getFaseAtual().finalizar();		
		if(ganhou) {
			jogo.passou();
		} else {
			jogo.perdeu();
		}		
	}
	
	public void pararExecucao() {
		parar = true;
	}
	
	public void continuarExecucao() {
		parar = false;
		notificar();
	}
	
	public synchronized void notificar() {
		notify();
	}
	
	private synchronized void esperar() {
		try {
			wait();
		} catch(InterruptedException e) {
			
		}
	}

	public boolean isAtravessarParedes() {
		return jogo.isAtravessarParedes();
	}

	public boolean isMudarFase() {
		return mudarFase;
	}

	public void setMudarFase(boolean mudarFase) {
		this.mudarFase = mudarFase;
	}

	public boolean isParar() {
		return parar;
	}

	public void setParar(boolean parar) {
		this.parar = parar;
	}
	
}
