package italo.comecome.personagem;

import italo.comecome.a_estrela.CelulaAEstrela;
import italo.comecome.logica.Fase;
import italo.comecome.logica.Jogo;
import italo.comecome.logica.Tabuleiro;

import java.util.List;


public class Monstrinho extends Personagem {
			
	public final static int FORTE = 1;
	public final static int MEIO_FRACO = 2;
	public final static int FRACO = 3;
	public final static int MUITO_FRACO = 4;

	private int estado = FORTE;
	private boolean fim = false;
	private boolean parar = false;
	private boolean finalizou = false;
	private List<CelulaAEstrela> caminhoAEstrela;
	
	private MonstrinhoIT monstrinhoIMP = new MonstrinhoIMP(jogo);
	
	public Monstrinho(Jogo jogo, String nome, int id, int sleep) {
		super(jogo, nome, id, sleep);
	}

	public void executar() {
		while(!fim) {
			while(parar)
				esperar();
			monstrinhoIMP.tratarEstado(this);
			monstrinhoIMP.mover(this);
			if(!fim && !parar)
				monstrinhoIMP.dormir(this);
		}				
		setFinalizou(true);
	}
	
	public void iniciar() {
		setFim(false);
		setParar(false);
		setEstado(FORTE);
		setFinalizou(false);		
	}
	
	public boolean podeMover() {
		Fase fase = jogo.getFaseAtual();		
		if(fase == null) 
			return false;		
		boolean podeMover = true;
		if(!fase.podeMover(this)) {
			Tabuleiro tabuleiro = jogo.getFaseAtual().getTabuleiro();
			int xH = getPosicao().getX()/tabuleiro.getLarguraCelula();
			int yH = getPosicao().getY()/tabuleiro.getAlturaCelula();
			if(!fase.podeSairDaCasa(xH, yH)) {
				boolean podeEntrar = false;
				int estadoMonstrinho = getEstado();
					if(estadoMonstrinho == Monstrinho.MUITO_FRACO)
						podeEntrar = fase.podeEntrarNaCasa(xH, yH);						
				podeMover = podeEntrar;
			}
		}
		return podeMover;
	}

	public void pararExecucao() {
		parar = true;
	}
	
	public void continuarExecucao() {
		parar = false;
		notificar();
	}
	
	public synchronized void notificar() {
		notify();
	}
	
	private synchronized void esperar() {
		try {
			wait();
		} catch(InterruptedException e) {
			
		}
	}
	
	public boolean isFim() {
		return fim;
	}

	public void setFim(boolean fim) {
		this.fim = fim;
	}

	public int getEstado() {
		return estado;
	}

	public void setEstado(int estado) {
		this.estado = estado;
	}

	public boolean isParar() {
		return parar;
	}

	public boolean isFinalizou() {
		return finalizou;
	}

	public void setFinalizou(boolean finalizou) {
		this.finalizou = finalizou;
	}

	public List<CelulaAEstrela> getCaminhoAEstrela() {
		return caminhoAEstrela;
	}

	public void setCaminhoAEstrela(List<CelulaAEstrela> caminhoAEstrela) {
		this.caminhoAEstrela = caminhoAEstrela;
	}

	public void setParar(boolean parar) {
		this.parar = parar;
	}
		
}
