package italo.comecome.personagem;

import italo.comecome.logica.Fase;
import italo.comecome.logica.Jogo;

public abstract class Personagem implements PersonagemIT {

	public final static int FRENTE = 1;
	public final static int BAIXO = 2;
	public final static int TRAZ = 3;
	public final static int CIMA = 4;
	
	public final static int DIRECAO_MINIMA = 1;
	public final static int DIRECAO_MAXIMA = 4;
				
	protected String nome;
	protected int id;
	protected int sleep = 50;	
	protected int direcao = FRENTE;
	protected int novaDirecao = FRENTE;
	protected Posicao posicao;
	protected Jogo jogo;	
		
	public Personagem(Jogo jogo, String nome, int id, int sleep) {
		this.jogo = jogo;
		this.nome = nome;
		this.id = id;
		this.sleep = sleep;
	}
			
	public abstract boolean podeMover();
	public abstract void iniciar();
		
	public void mover() {		
		if(podeMover()) {			
			Fase fase = jogo.getFaseAtual();			
			int direcao = getDirecao();			
			int novaDirecao = getNovaDirecao();
			Posicao posicao = getPosicao();			
			if(direcao != novaDirecao && fase.podeMoverParaNovaDirecao(this)) 
				setDirecao(novaDirecao);		
			direcao = getDirecao();			
			setPosicao(fase.criarPosicaoNova(direcao, posicao));					
		}				
		jogo.getFaseAtual().mover(this);	
	}

	public Posicao getPosicao() {
		return posicao;
	}

	public void setPosicao(int x, int y) {
		this.posicao = new Posicao(x, y);
	}
	
	public void setPosicao(Posicao p) {
		this.posicao = p;
	}
	
	public int getDirecao() {
		return direcao;
	}

	public void setDirecao(int direcao) {
		this.direcao = direcao;
	}

	public int getNovaDirecao() {
		return novaDirecao;
	}

	public void setNovaDirecao(int novaDirecao) {
		this.novaDirecao = novaDirecao;
	}

	public String getNome() {
		return nome;
	}
	
	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public int getSleep() {
		return sleep;
	}

	public void setSleep(int sleep) {
		this.sleep = sleep;
	}

	public void setNome(String nome) {
		this.nome = nome;
	}
				
}
